package com.bgy.sms.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bgy.sms.channel.bgy.config.BgySMSConfig;
import com.bgy.sms.channel.bgy.dto.CLBizResponse;
import com.bgy.sms.channel.bgy.service.BgySmsService;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.channel.dto.*;
import com.bgy.sms.repository.domain.DmBatchInfo;
import com.bgy.sms.repository.domain.SmsTemplateInfo;
import com.bgy.sms.repository.domain.SysBatchInfo;
import com.bgy.sms.service.DmBatchService;
import com.bgy.sms.service.MessageService;
import com.bgy.sms.service.SmsTemplateService;
import com.bgy.sms.service.bean.TemplateChangeBean;
import com.bgy.util.id.IdHandler;
import com.sun.org.apache.regexp.internal.RE;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@SuppressWarnings("Duplicates")
@Service
public class MessageServiceImpl implements MessageService {

    private static final Logger log = LoggerFactory.getLogger(MessageServiceImpl.class);

    @Autowired
    private SysBatchServiceImpl sysBatchService;
    @Autowired
    private SmsTemplateService smsTemplateService;
    @Autowired
    private BgySmsService bgySmsService;
    @Autowired
    private DmBatchService dmBatchService;

    /**
     * 短信模板创建
     *
     * @param requestDTO
     * @return
     */
    @Override
    public DmHubResponse createTemplate(DmHubTemplateRequest requestDTO) {
        log.info("进入创建短信模板业务逻辑，入参：{}", requestDTO);

        DmHubResponse responseDTO = new DmHubResponse(ResponseCode.SUCCESS);
        try {
            String templateContent = requestDTO.getTemplateContent();
            String signature = requestDTO.getSignature();
            String smsType = requestDTO.getSmsType();
            String tenantId = requestDTO.getTenantId();
            String templateId = requestDTO.getTemplateId();
            String templateName = requestDTO.getTemplateName();

            // 替换模板格式，保存适合碧桂园发送短信的模板，避免发送时修改模板格式
            TemplateChangeBean bean = dmHub2BgyTemplateSend(templateContent);
            String chuanglanSend = bean.getUpSendStr();
            String params = bean.getParams().toString();
            SmsTemplateInfo dbInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
            if (dbInfo != null) {
                log.error("Dm Hub请求创建模板，模板id已存在，模板id：【{}】", templateId);
                return new DmHubResponse(ResponseCode.TEMPLATE_ALREADY_EXISTED);
            }
            SmsTemplateInfo info = new SmsTemplateInfo();
            info.setContent(templateContent);
            info.setType(smsType);
            info.setDmTemplateId(templateId);
            info.setTenantId(tenantId);
            info.setId(IdHandler.nextId());
            info.setSignature(signature);
            info.setTemplateName(templateName);
            info.setDateCreated(new Date());
            info.setLastUpdated(new Date());
            info.setLastUpdated(new Date());
            info.setUpContent(chuanglanSend);
            info.setParams(params);
            boolean insert = smsTemplateService.insert(info);
            if (!insert) {
                //TODO 发送告警
                log.error("模板插入DB异常：【{}】", JSONObject.toJSONString(info));
                return new DmHubResponse(ResponseCode.SYSTEM_ERROR);
            }
        } catch (Exception exception) {
            log.error("创建模板业务逻辑异常，错误信息", exception);
            responseDTO = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        return responseDTO;

    }


    /**
     * 将DM HUb的模板装成可以简单模式模板，用户后续参数封装
     *
     * @param content
     * @return
     */
    private TemplateChangeBean dmHub2BgyTemplateSend(String content) {
        //正则获取${***} 和 ${surl***'}格式的内容，其中：((?:\$\{surl.*'\}))獲取${surl***'}格式，\$\{[^}]+\} 获取${***}格式，顺序不能变，否则不能完成匹配格式一
        String regex = "((?:\\$\\{surl.*'\\}))|\\$\\{[^}]+\\}";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(content);

        List<String> groups = new ArrayList<>();
        while (matcher.find()) {
            String group = matcher.group();
            groups.add(group);
        }
        JSONArray arr = new JSONArray();
        if (groups.isEmpty()) {
            return new TemplateChangeBean(content, arr, content);
        }

        String upCreateStr = content;
        String upSendStr = content;
        for (String g : groups) {
            String t = g;
            g = Matcher.quoteReplacement(g);
            g = escapeRegex(g);
            upSendStr = upSendStr.replaceAll(g, "\\{\\$var\\}");
            if (t.contains("${surl!")) {
                // 短链没传长度，固定长度20
                upCreateStr = upCreateStr.replaceAll(g, "\\{s20\\}");
            } else {
                int i = t.indexOf("@");
                int i1 = t.indexOf("!");
                String str = t.substring(i + 1, i1);
                if (isNumeric(str)) {
                    Integer integer = Integer.valueOf(str);
                    if (integer == 0) {
                        upCreateStr = upCreateStr.replaceAll(g, "\\{s20\\}");
                    } else {
                        upCreateStr = upCreateStr.replaceAll(g, "\\{s" + integer + "\\}");
                    }
                } else {
                    upCreateStr = upCreateStr.replaceAll(g, "\\{s20\\}");
                }
            }
        }
        arr.addAll(groups);
        return new TemplateChangeBean(upSendStr, arr, upCreateStr);
    }

    private boolean isNumeric(String str) {
        if (StringUtils.isBlank(str)) {
            return false;
        }
        for (int i = 0; i < str.length(); i++) {
            System.out.println(str.charAt(i));
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    private static String escapeRegex(String s) {
        String[] symbols = new String[]{"\\/", "\\[", "\\]", "\\(", "\\)", "\\{", "\\}", "\\?", "\\+", "\\*", "\\|", "\\.", "\\^", "\\-", "\\^"};
        if (s != null) {
            for (String sym : symbols) {
                s = s.replaceAll(sym, "\\" + sym);
            }
        }
        return s;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public DmHubResponse send(DmHubSendRequest request) {
        try {
            String dmHubBatchId = request.getBatchId();
            String templateId = request.getTemplateId();
            String audienceIdType = request.getAudienceIdType();
            JSONObject data = request.getData();
            // 1、获取模板信息
            SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
            if (null == templateInfo) {
                log.error("未获取到短信模板");
                return new DmHubResponse("999", "未获取到短信模板");
            }
            // 2、根据请求信息获取用户的手机号码（此处不考虑配置DMHUB系统不是SMS的情况），直接拿请求体中的_audienceId字段值，该值为手机号码
            // 模板参数占位符
            String params = templateInfo.getParams();
            JSONArray paramsArr = JSONArray.parseArray(params);
            String smsType = templateInfo.getType();
            // 封装参数
            List<String> paramList = new ArrayList<>();
            String mobile = data.getString("_audienceId");
            data.remove("name");
            data.remove("id");
            data.remove("_audienceId");

            log.info("data信息：{}", data);
            if (!data.isEmpty()) {
                List<String> paramsList = new ArrayList<>();
                StringBuffer sb = new StringBuffer();
                Map<String, String> retMap = new HashMap<>();
                getKeyAndValue(data, paramsList, sb, retMap);
                for (Object parm : paramsArr) {
                    String s = parm.toString();
                    for (String newKeyStr : paramsList) {
                        String newKey = newKeyStr.substring(1);
                        if (s.contains(newKey)) {
                            paramList.add(retMap.get(newKeyStr));
                        }
                    }
                }
            }
            String type = templateInfo.getType();
            // 3、记录批次信息
            SysBatchInfo sysBatchInfo = new SysBatchInfo();
            long sysBatchId = IdHandler.nextId();
            sysBatchInfo.setBatchId(sysBatchId);
            sysBatchInfo.setDmBatchId(dmHubBatchId);
            sysBatchInfo.setId(IdHandler.nextId());
            sysBatchInfo.setDmTemplateId(templateId);
            sysBatchInfo.setReport("1");
            sysBatchInfo.setSmsNum(1);
            sysBatchInfo.setDateCreated(new Date());
            sysBatchInfo.setLastUpdated(new Date());
            boolean insert = sysBatchService.insert(sysBatchInfo);

            CLBizResponse response = new CLBizResponse();
            String content = null;
            if (paramsArr == null || paramsArr.isEmpty()) {
                // 固定短信发送
                content = templateInfo.getUpContent();
            } else {
                // 变量短信发送
                String upContent = templateInfo.getUpContent();
                content = getMsg(upContent, paramList);
            }
            response = bgySmsService.sendSms(mobile, content);
            String code = response.getCode();
            if (code.equals(ResponseCode.SUCCESS.getCode())) {
                return new DmHubResponse(ResponseCode.SUCCESS);
            } else {
                return new DmHubResponse(code, response.getMsg());
            }

        } catch (Exception e) {
            log.error("调用短信发送逻辑错误", e);
            return new DmHubResponse("999", "插件系统异常");
        }
    }

    @Override
    public DmHubResponse sendCode(DmHubCodeRequest params) {
        String mobile = params.getMobile();
        String code = params.getCode();
        CLBizResponse response = new CLBizResponse();
        String content = "您的验证码是：" + code;
        try {
            response = bgySmsService.sendSms(mobile, content);
            String retCode = response.getCode();
            if (retCode.equals(ResponseCode.SUCCESS.getCode())) {
                return new DmHubResponse(ResponseCode.SUCCESS);
            } else {
                return new DmHubResponse(retCode, response.getMsg());
            }
        } catch (Exception e) {
            log.error("发送短信异常：", e);
            return new DmHubResponse("999", "发送通道异常");
        }

    }

    /**
     * 获取可以直接用户发送的短信内容
     *
     * @param upContent
     * @param params
     * @return
     */
    private String getMsg(String upContent, List<String> params) {
        for (String info : params) {
            upContent = upContent.replaceFirst("\\{\\$var\\}", info);
        }
        return upContent;
    }

    @Override
    public DmHubResponse batchSend(DmHubBatchSendRequest request) {
        String batchId = request.getBatchId(); // DM hub 批次号
        String templateId = request.getTemplateId(); // 此次短信对应的模板id
        String audienceIdType = request.getAudienceIdType();
        List<JSONObject> data = request.getData();

        SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
        if (null == templateInfo) {
            return new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        // 模板参数占位符
        String params = templateInfo.getParams();
        JSONArray paramsArr = JSONArray.parseArray(params);

        // 封装参数
//        StringBuilder sendParams = new StringBuilder();

        Map<String, List<String>> paramsMap = new HashMap();
        int i = 0;
        for (JSONObject json : data) {
            i++;
            Set<String> keys = json.keySet();
            String mobile = json.getString("_audienceId");
            json.remove("name");
            json.remove("id");
            json.remove("_audienceId");

            if (!json.isEmpty()) {
                for (Object parm : paramsArr) {
                    List<String> list = new ArrayList<>();
                    String s = parm.toString();
                    for (String key : keys) {
                        String newKey = key;
                        Object value = json.get(key);
                        if (value instanceof JSONObject || value instanceof Map) {
                            StringBuilder keyBuffer = new StringBuilder(key);
                            value = getKeyValue(value, keyBuffer);
                            newKey = keyBuffer.toString();
                        }
                        if (s.contains(newKey)) {
                            list.add(value.toString());
                        }
                    }
                    paramsMap.put(mobile, list);
                }

            } else {
                if (i != data.size()) {
                    paramsMap.put(mobile, new ArrayList<>());
                }
            }
        }

        DmBatchInfo dmInfo = new DmBatchInfo();
        dmInfo.setDmBatchId(batchId);
        dmInfo.setDmTemplateId(templateId);
        dmInfo.setSmsNum(data.size());
        dmInfo.setId(IdHandler.nextId());
        dmInfo.setDateCreated(new Date());
        dmInfo.setLastUpdated(new Date());
        dmBatchService.insert(dmInfo);

        SysBatchInfo info = new SysBatchInfo();
        info.setId(info.getBatchId());
        info.setDmBatchId(batchId);
        info.setSmsNum(data.size());
        info.setDmTemplateId(templateId);
        Long sysBatchId = IdHandler.nextId();
        info.setBatchId(sysBatchId);
        info.setDateCreated(new Date());
        info.setLastUpdated(new Date());
        sysBatchService.insert(info);
        CLBizResponse response = new CLBizResponse();


        if (paramsArr == null || paramsArr.isEmpty()) {
            try {
                String code = response.getCode();
                String msg = response.getMsg();
                // TODO 根据DM hub需要的返回数据封装
                if (ResponseCode.SUCCESS.getCode().equals(code)) {
                    return new DmHubResponse(ResponseCode.SUCCESS);
                } else {
                    return new DmHubResponse(code, msg);
                }
            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误", e);
                // TODO 根据DM hub需要的返回数据封装
                return new DmHubResponse(ResponseCode.SYSTEM_ERROR);
            }
        } else {
            // 固定短信
            Set<Map.Entry<String, List<String>>> entries = paramsMap.entrySet();
            try {
                // 此处不做拆分操作
                for (Map.Entry<String, List<String>> entry : entries) {
                    String mobile = entry.getKey().toString();
                    List<String> paramsList = entry.getValue();
                    String sendMsg = getMsg(templateInfo.getUpContent(), paramsList);
                    response = bgySmsService.sendSms(mobile, sendMsg);
                }
                return new DmHubResponse(ResponseCode.SUCCESS);
            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误:", e);
                return new DmHubResponse(ResponseCode.SYSTEM_ERROR);
            }
        }
    }

    @Override
    public DmHubResponse batchSendOneByOne(DmHubBatchSendRequest request) {
        String batchId = request.getBatchId(); // DM hub 批次号
        String templateId = request.getTemplateId(); // 此次短信对应的模板id
        String audienceIdType = request.getAudienceIdType();
        List<JSONObject> data = request.getData();

        SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
        if (null == templateInfo) {
            return new DmHubResponse("999", "短信插件未获取到模板信息");
        }
        String smsType = templateInfo.getType();
        // 模板参数占位符
        String params = templateInfo.getParams();
        JSONArray paramsArr = JSONArray.parseArray(params);

        // 封装参数
        List<String> mobileList = new ArrayList<>();
        Map<String, List<String>> paramsMap = new HashMap();
        int i = 0;
        for (JSONObject json : data) {
            i++;
            Set<String> keys = json.keySet();
            String mobile = json.getString("_audienceId");
            json.remove("name");
            json.remove("id");
            json.remove("_audienceId");
            mobileList.add(mobile);
            List<String> list = new ArrayList<>();
            if (!json.isEmpty()) {
                List<String> paramsList = new ArrayList<>();
                StringBuffer sb = new StringBuffer();
                Map<String, String> retMap = new HashMap<>();
                getKeyAndValue(json, paramsList, sb, retMap);
                for (Object parm : paramsArr) {
                    String s = parm.toString();
                    for (String newKeyStr : paramsList) {
                        String newKey = newKeyStr.substring(1);
                        if (s.contains(newKey)) {
                            list.add(retMap.get(newKeyStr).toString());
                        }
                    }
                    paramsMap.put(mobile, list);
                }

            } else {
                if (i != data.size()) {
                    paramsMap.put(mobile, new ArrayList<>());
                }
            }
        }


        DmBatchInfo dmInfo = new DmBatchInfo();
        dmInfo.setDmBatchId(batchId);
        dmInfo.setDmTemplateId(templateId);
        dmInfo.setSmsNum(data.size());
        dmInfo.setId(IdHandler.nextId());
        dmInfo.setDateCreated(new Date());
        dmInfo.setLastUpdated(new Date());
        dmBatchService.insert(dmInfo);

        SysBatchInfo info = new SysBatchInfo();
        info.setId(info.getBatchId());
        info.setDmBatchId(batchId);
        info.setSmsNum(data.size());
        info.setDmTemplateId(templateId);
        Long sysBatchId = IdHandler.nextId();
        info.setBatchId(sysBatchId);
        info.setDateCreated(new Date());
        info.setLastUpdated(new Date());
        sysBatchService.insert(info);
        CLBizResponse response = new CLBizResponse();


        if (paramsArr == null || paramsArr.isEmpty()) {
            try {
                for (String mobile : mobileList) {
                    try {
                        response = bgySmsService.sendSms(mobile, templateInfo.getUpContent());
                    } catch (Exception e) {
                        log.error("短信发送异常", e);
                    }
                }
            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误", e);
                // TODO 根据DM hub需要的返回数据封装
                return new DmHubResponse("999", "插件服务系统异常");
            }
        } else {
            // 变量短信
            Set<Map.Entry<String, List<String>>> entries = paramsMap.entrySet();
            try {
                for (Map.Entry<String, List<String>> entry : entries) {
                    try {
                        String mobile = entry.getKey().toString();
                        List<String> paramsList = entry.getValue();
                        String sendMsg = getMsg(templateInfo.getUpContent(), paramsList);
                        response = bgySmsService.sendSms(mobile, sendMsg);
                        String code = response.getCode();
                        String msg = response.getMsg();
                    } catch (Exception e) {
                        log.error("chu0");
                    }

                }
                // TODO 待优化
                return new DmHubResponse(ResponseCode.SUCCESS);
            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误:", e);
                return new DmHubResponse("999", "插件服务系统异常");
            }
        }
        return new DmHubResponse(ResponseCode.SUCCESS);
    }

    private static void getKeyAndValue(JSONObject json, List<String> keyList, StringBuffer keyBuffer, Map<String, String> paramsMap) {
        Set<String> keys = json.keySet();
        for (String key : keys) {
            keyBuffer.append(".").append(key);
            Object info = json.get(key);
            StringBuffer tempInfo = new StringBuffer();

            if (info instanceof JSONObject || info instanceof Map) {
                String s = JSONObject.toJSONString(info);
                JSONObject valueJson = JSONObject.parseObject(s);
                getKeyAndValue(valueJson, keyList, keyBuffer, paramsMap);
                int key1 = keyBuffer.indexOf(key);
                String substring = keyBuffer.substring(0, key1 - 1);
                System.out.println(substring);
                keyBuffer = new StringBuffer(substring);
            } else {
                paramsMap.put(keyBuffer.toString(), info.toString());
                keyList.add(keyBuffer.toString());
                int key1 = keyBuffer.indexOf(key);
                String substring = keyBuffer.substring(0, key1 - 1);
                System.out.println(substring);
                keyBuffer = new StringBuffer(substring);
            }
        }
    }


    private String getKeyValue(Object value, StringBuilder keyBuffer) {
        String s = JSONObject.toJSONString(value);
        JSONObject valueJson = JSONObject.parseObject(s);
        Set<String> set = valueJson.keySet();
        if (set.isEmpty()) {
            return value.toString();
        }
        for (String str : set) {
            keyBuffer.append(".").append(str);
            Object info = valueJson.get(str);
            if (info instanceof JSONObject || info instanceof Map) {
                String test = getKeyValue(info, keyBuffer);
                return test;
            } else {
                return info.toString();
            }
        }
        return value.toString();
    }


}


